/* marvell_phy.c -  contains apis to initialize and access PHY dev */

/*  Copyright 2005-2006 Freescale Semiconductor */
/*
modification history
--------------------
01a,19jul05,ahk written
*/

/*
DESCRIPTION
This module defines the APIs that will initialize the PHY device
over the management interface. It contains all the sequence of 
operation required to bring the PHY to a working state - for 
digital internal PHY loopback and also for talking to the external
world.

INCLUDE Files: MPC8560.h, tsec.h
*/

/* includes */

#include <stdio.h>
#include "MPC8560.h"
#include "tsec.h"

/* defines */

/* typedefs */


/* globals */

void phyInit (void);
void phyReset (void);
void phyId (void);
unsigned long autoNegotiate (void);


/******************************************************************
* phyInit - initializes the PHY device
*
* This routine initializes the PHY device via the management interface
* and make the phy ready for data transfer over either GMII or MII or
* TBI interface. Note that most of the configuration values are obtained
* by the PHY device from the CONFIGx pins of the device (Marvell, Alaska
* Ultra). Only few parameters need to be changed in order to get the
* PHY digital loopback to work. For more information on the configuration
* of the PHY device, refer to MPC8560ADS schematic and user's manaul. You
* can also refer to Alaska user's manual. 
*
* RETURNS: None
*      
* ERRNO: None 
*
*/

void phyInit (void)
{
	unsigned long i;
	unsigned long control;
	unsigned long status;
	unsigned long regValue;

	/* Initialize the GMII management interface */
	
	mgmtInit ();
	
	/* 
	* Read the PHY ids from register 2 and 3.This code tests 
	* that the management interface is being initialized properly. 
	* The read value from these two  registers should match with 
	* the known values of the PHY ids.  
	*/
	
	phyId ();
	
	/* 
	* If loopback mode is requested then loopback field, the
	* duplex control field and the speed control field of the phy
	* is hardcoded. In normal mode of operation auto negotiation is
	* done
	*/
    #ifdef NORMAL_MODE

    /* Initiate auto negotiation */
    
    status = autoNegotiate ();
    
    //regValue = (REG0_15_DUPLEX_FIELD   | 
    //            REG0_15_GIGASPEED_FIELD);    
    #endif


	#ifdef PHY_LOOPBACK

    /* Write to control register (0) */

    regValue = (REG0_14_LOOPBACK_FIELD | 
                REG0_15_DUPLEX_FIELD   | 
                REG0_15_GIGASPEED_FIELD);

	mgmtWrite (TSEC1_PHY_ADDR, REG0_MARVELL_PHY, regValue);

	#endif
                
    #ifdef PHY_CONNECTOR_LOOPBACK

    /* 
    * FIXME : The connector loopback didn't work.
    * Need to figure out what registers in the phy needed
    * to be programmed in order to do that. 
    */
    
	mgmtWrite (TSEC1_PHY_ADDR, REG18_MARVELL_PHY, 0x0);
	mgmtWrite (TSEC1_PHY_ADDR, REG9_MARVELL_PHY, 0x1b00);
	mgmtWrite (TSEC1_PHY_ADDR, REG0_MARVELL_PHY, 0x9140);		

	/*
	* The following register offsets are not defined
	* in the marvell phy for pq3 ads board. The initialization
	* values are taken from FIL TSEC driver for MPC8349ADS.
	* But the PHY used in the MPC8349 board is diff. than 
	* what we use on the PQ3 ADS. These values are put here
	* just to blindly check whether this code would work.
	*/
	/*
	mgmtWrite (TSEC1_PHY_ADDR, 0x1d, 0x7);			
	mgmtWrite (TSEC1_PHY_ADDR, 0x1e, 0x808);

	mgmtWrite (TSEC1_PHY_ADDR, 0x1d, 0x10);			
	mgmtWrite (TSEC1_PHY_ADDR, 0x1e, 0x42);
	
	mgmtWrite (TSEC1_PHY_ADDR, 0x1d, 0x12);			
	mgmtWrite (TSEC1_PHY_ADDR, 0x1e, 0x8901);	
	*/

	/*
	        f_PhyWriteReg(miiMngConfigCntrlBaseAddr,phyAdr,0x12,0x0,0);
            f_PhyWriteReg(miiMngConfigCntrlBaseAddr,phyAdr,0x9,0x1b00,0);
            f_PhyWriteReg(miiMngConfigCntrlBaseAddr,phyAdr,0x0,0x9140,0);

            f_PhyWriteReg(miiMngConfigCntrlBaseAddr,phyAdr,0x1d,0x7,0);
            f_PhyWriteReg(miiMngConfigCntrlBaseAddr,phyAdr,0x1e,0x808,0);

            f_PhyWriteReg(miiMngConfigCntrlBaseAddr,phyAdr,0x1d,0x10,0);
            f_PhyWriteReg(miiMngConfigCntrlBaseAddr,phyAdr,0x1e,0x42,0);
            
            f_PhyWriteReg(miiMngConfigCntrlBaseAddr,phyAdr,0x1d,0x12,0);
            f_PhyWriteReg(miiMngConfigCntrlBaseAddr,phyAdr,0x1e,0x8901,0);
    */

    #endif


	
	/* Read the control register */
	
	control = mgmtRead (TSEC1_PHY_ADDR, REG0_MARVELL_PHY);
	
	/* 
	* Read the control register one more time. This
	* is needed because the PHY reference manual states
	* that in order to get the correct link status this
	* register needs to be read back-to-back
	*/
	
	control = mgmtRead (TSEC1_PHY_ADDR, REG0_MARVELL_PHY);	

	/* Read the status register (1) */
	
	status = mgmtRead (TSEC1_PHY_ADDR, REG1_MARVELL_PHY);	
	
	/* Read the phy status register (17) */
	
	status = mgmtRead (TSEC1_PHY_ADDR, REG17_MARVELL_PHY);		
	
	return;
}

/******************************************************************
* phyReset - resets the PHY device
*
* This routine resets the PHY device. We don't use this routine
* for the initial release. We rely upon the default config values
* of the PHY device upon power on reset. We checked these values 
* for a scenario where config plug is connected to J55/J47, and
* also for a scenario where config plug is not connected to J55/J47.
* With or without the config plug scenario yields working default
* values for the PHY. One thing to note is that with config plug 
* on at J55 for TSEC1, the phy addr is 15, whereas without the config
* plug the phy addr is 0. In this test code it is assumed that the
* board doesn't have a config plug at J55.
*
* RETURNS: None
*      
* ERRNO: None 
*
*/

void phyReset (void)
{
	unsigned long value;
	
	/* now reset the phy device */	
	mgmtWrite (TSEC1_PHY_ADDR,REG0_MARVELL_PHY,0x00008000);
	
	/* wait for the reset to complete */
	while(1)
	{
		value = mgmtRead (TSEC1_PHY_ADDR, REG0_MARVELL_PHY);	
		if (!(value & 0x80000000))
			break;
	}

}

/******************************************************************
* phyId - provides the ids of the phy device
*
* This routine provides the ids of the phy device. It is useful
* to invoke this routine for debug purpose of the management 
* interface. From the reference manual we know that register
* 2 should provide an id value of 0x0141 and register 3 should
* provide an id value of 0x0c26. 
*
* RETURNS: None
*      
* ERRNO: None 
*
*/

void phyId (void)
{
	unsigned long value1;
	unsigned long value2;

	/* Read marvell id from reg 2.This test the mgmt interface */
	
	value1 = mgmtRead (TSEC1_PHY_ADDR, REG2_MARVELL_PHY);
	asm("isync");
	asm("msync");	

	/* Read marvell id from reg 3.This test the mgmt interface */	
	
	value2 = mgmtRead (TSEC1_PHY_ADDR, REG3_MARVELL_PHY);
	asm("isync");
	asm("msync");
	
}


unsigned long autoNegotiate (void)
{
	unsigned long regValue;
	unsigned long readPhyReg;
	unsigned long timeout = 1000000;
	unsigned long status;
	unsigned long control;
	
	/* Write 0x1340 to control register (0) */
	
	regValue = 0x1340;
	mgmtWrite (TSEC1_PHY_ADDR, REG0_MARVELL_PHY, regValue);	
	
	/* Read back the control register for debug purpose */
	
	control = mgmtRead (TSEC1_PHY_ADDR, REG0_MARVELL_PHY);
	
	/* 
	* Read from status register (1) Check to see whether 
	* AN is done. Compare the value 0x0020. That is check whether 
	* bit 1.3 is set or not. If set this indicates that phy was 
	* able to perform auto negotiation 
	*/
	
	while (1)
	{
		readPhyReg = mgmtRead (TSEC1_PHY_ADDR, REG1_MARVELL_PHY);
		if (readPhyReg & 0x0020)
			break;
	}

	
	/* 
	* Check the speed and duplex mode of phy. Reg
	* 17.11 is the status register in the phy. If this
	* bit is set that indicates speed and duplex was
	* resolved 
	*/
	
	while (1)
	{
		readPhyReg = mgmtRead (TSEC1_PHY_ADDR, REG17_MARVELL_PHY);
		if (readPhyReg & 0x0800)
			break;
	}
	

	
	/*
	* AN is succeful and therefore we need to configure
	* the MAC accordingly. Program the MACCFG2 register
	* to set speed and duplex mode of the MAC.
	*/
	
	pTsecMap->maccfg2     |= (MACCFG2_GMII_TBI_MODE | MACCFG2_FULL_DUPLEX ); 	                        			
	
	return 0;	
}