/* mgmt.c - apis that implement management init/read/write */

/*  Copyright 2005-2006 Freescale Semiconductor */
/*
modification history
--------------------
01a,19jul05,ahk written
*/

/*
DESCRIPTION
This module defines the APIs that will configure the management
interface and also it defines the management reads and writes 
operation. Note that unlike CPM FCC, TSEC controller has dedicated
registers part of its internal memory map, which is used to 
implement the management apis.   

INCLUDE Files: MPC8560.h, tsec.h
*/

/* includes */

#include <stdio.h>
#include "MPC8560.h"
#include "tsec.h"

/* defines */



/* typedefs */


/* globals */

void mgmtInit(void);
void mgmtWrite (unsigned long phy_addr, unsigned long reg_addr, unsigned long data);
unsigned long mgmtRead(unsigned long phy_addr, unsigned long reg_addr);

/******************************************************************
* mgmtInit - initializes the management interface of given TSEC
*
* This routine initializes the management interface. After initialization
* is performed the controller can do reads and writes using the 
* management interface to read out data and write into data respectively
* to the PHY device.
*
* RETURNS: None
*      
* ERRNO: None 
*
*/

void mgmtInit (void)
{
	unsigned long phy_addr;
	unsigned long reg_addr;

	/* 
	* Assign the phy addr to the tbi interface for both TSEC1 and TSEC2.
	* We must assign a PHY address for the TBI interface in order for
	* the management interface to work properly with the existing PHYs.
	*/

	pTsecMap->tbipa = 0x1f;
   	
	/* 
	* CHECK : Our user manual states that if bit 30 of MIIMCFG register
	* is set then MII reset will occur. But it doesn't describe how to become
	* sure that the management interface was really reset succesfully or not.
	* Any how the current scheme 
	*/
	
	pTsecMap->miimcfg = MIIMCFG_RESET; 

	/* set up the MII management clock speed - set to divide by 14 -> (266/8)/14 = 2.375 MHz */
	pTsecMap->miimcfg = MIIMCFG_CLOCK_DIV_14;

	while(pTsecMap->miimind & 0x00000001)
	{
		
	}
	
	#if 0
	/* 
	* CHECK : Perform a dummy mii read from from a PHY register
	* in order to stabilize the management interface. This is 
	* suggested by Michaell Reiss at Toronto. Still need to dig
	* out whether this is really needed or not. 
	*/
	
	phy_addr = 0;
	reg_addr = 0;
	pTsecMap->miimadd = ((phy_addr << 8) | reg_addr);
	asm("isync");
	asm("msync");
		
	pTsecMap->miimcom &= 0xfffffffe;
	asm("isync");
	asm("msync");	
	
	pTsecMap->miimcom |= 0x00000001;
	asm("isync");
	asm("msync");
	
	while ((pTsecMap->miimind & 0x00000001) && (pTsecMap->miimind & 0x00000004))
	{
		
	}
	asm("isync");
	asm("msync");
	#endif 
	
	return;
}

/******************************************************************
* mgmtWrite - writes the given data to PHY reg using MDC and MDIO
*
* This routine puts the user provided data into the PHY register.
* Check is performed to make sure that the write is completed
*
* RETURNS: None
*      
* ERRNO: None 
*
*/

void mgmtWrite (unsigned long phy_addr, unsigned long reg_addr, unsigned long data)
{
	unsigned long timeout = 1000000;
	pTsecMap->miimadd = ((phy_addr << 8) | reg_addr);
	pTsecMap->miimcon = data;	
	
	/* 
	* This msync is not required. Tested it without
	* the msync and the loopback worked.
	*/
	
	asm("msync");
	
	while ((pTsecMap->miimind & 0x00000001) && timeout--)
	{
		
	}
	return;
}

/******************************************************************
* mgmtRead - reads data out from PHY reg using MDC and MDIO
*
* This routine reads data out from a given PHY register. It checks
* to make sure that the read is completed.
*
* RETURNS: data from PHY register
*      
* ERRNO: None 
*
*/

unsigned long mgmtRead 
	(
 	unsigned long phy_addr,  /* phy addr determined by h/w config of PHY */
 	unsigned long reg_addr   /* register of the PHY */
	)
	{
		unsigned long value;
		pTsecMap->miimadd = ((phy_addr << 8) | reg_addr);

		/* 
		* Seems that the msync are not required but
		* put them anyways to make sure that the ordering of the
		* load-load, store-load etc doesn't kill our algorithm
		*/
		
		asm("msync");

		pTsecMap->miimcom = 0x0;
		asm("msync");

		pTsecMap->miimcom = 0x00000001;
		asm("msync");
		while ((pTsecMap->miimind & (0x00000001|0x00000004)))
			{
			}
		value = pTsecMap->miimstat;
		return value;
	}


