/* tsec.c - main subroutine that calls TSEC apis */

/*  Copyright 2005-2006 Freescale Semiconductor */
/*
modification history
--------------------
01a,19jul05,ahk written
*/

/*
DESCRIPTION
This module illustrates how the TSEC controllers (TSEC1/TSEC2) of
MPC8560 can be programmed. It shows how the mac loopback works, how the 
PHY digital loopback works and it also shows how the TSEC controller 
interacts with the external world, that is generating and receiving Ethernet
traffic from external wires. The tested modes are GMII, MII, TBI. For 
PHY internal digital loopback it is assumed that we don't have config
plugs hooked up to either J55 or J47. See the header file tsec.h to 
know more about the phy ids and their relations to config plugs. Different
test cases are supported.
Before mentioning about the different test cases it is worth pointing out
that there are few board switch settings that you need to take care of 
depending upon how you want to configure the TSEC controller. In the 
following the notation SWn.m corresponds to switch number n and position
m on that swictch. Note that ON means logic value 0 and off means logic
value 1.
SW6.3 = TSEC_WIDTH = 0 -> RTBI/RGMII (4-bit bus)
                     1 -> TBI/GMII   (8-bit bus)
SW6.1 = TSEC1_CFG  = 0 -> GMII (width = 1) / RGMII (width = 0)
                     1 -> TBI (width = 1) / RTBI (width = 0)
SW6.2 = TSEC2_CFG =  0 -> GMII (width = 1) / RGMII (width = 0)
                     1 -> TBI (width = 1) / RTBI (width = 0)
                                        

Test Case 1: Mac internal loopback + GMII mode
-----------------------------------------------
SW6.3 = 1 = TSEC_WIDTH -> 8-bit bus
SW6.1 = 0 = TSEC1_CFG  -> GMII
define MAC_INTERNAL_LOOPBACK in tsec.h
undefine PHY_LOOPBACK in tsec.h
undefine NORMAL_MODE in tsec.h
define INTERFACE_MODE equals 	GMII_INTERFACE
TESTED and PASSED

Test Case 2: Mac internal loopback + RGMII mode
---------------------------------------------------
SW6.3 = 0 = TSEC_WIDTH -> 4-bit bus
SW6.1 = 0 = TSEC1_CFG  -> GMII => GMII and 4-bit = RGMII
define MAC_INTERNAL_LOOPBACK in tsec.h
undefine PHY_LOOPBACK in tsec.h
undefine NORMAL_MODE in tsec.h
define INTERFACE_MODE equals 	GMII_INTERFACE
TESTED and PASSED


Test Case 4: Mac internal loopback + TBI mode
-----------------------------------------------
SW6.3 = 1 = TSEC_WIDTH -> 8-bit bus
SW6.1 = 1 = TSEC1_CFG  -> TBI
define MAC_INTERNAL_LOOPBACK in tsec.h
undefine PHY_LOOPBACK in tsec.h
undefine NORMAL_MODE in tsec.h
define INTERFACE_MODE equals 	TBI_INTERFACE
TESTED and PASSED


Test Case 5: Mac internal loopback + RTBI mode
-----------------------------------------------
SW6.3 = 0 = TSEC_WIDTH -> 4-bit bus
SW6.1 = 1 = TSEC1_CFG  -> TBI => TBI and 4-bit = RTBI
define MAC_INTERNAL_LOOPBACK in tsec.h
undefine PHY_LOOPBACK in tsec.h
undefine NORMAL_MODE in tsec.h
define INTERFACE_MODE equals 	TBI_INTERFACE
TESTED and PASSED

Test Case 6: PHY internal loopback + GMII mode
-----------------------------------------------
SW6.3 = 1 = TSEC_WIDTH -> 8-bit bus
SW6.1 = 0 = TSEC1_CFG  -> GMII
Make sure that there is no config plug on J55
undefine MAC_INTERNAL_LOOPBACK in tsec.h
define PHY_LOOPBACK in tsec.h
undefine NORMAL_MODE in tsec.h
define INTERFACE_MODE equals 	GMII_INTERFACE


INCLUDE Files: MPC8560.h, tsec.h
*/

/* includes */

#include <stdio.h>
#include <string.h>
#include "MPC8560.h"
#include "tsec.h"


/* defines */



/* typedefs */


/* globals */
//extern mpc_imm                 *pCCSRBAR; /* pointer to the ccsrbar */
//extern mpq_tsec				   *pTsecMap; /* pointer to tsec memory mapped registers */
TSEC_TXBD			    transmitBd[NUM_OF_BD_BUFFER] __attribute__ ((aligned (64))); /* transmit BD ring */
TSEC_RXBD			    receiveBd[NUM_OF_BD_BUFFER] __attribute__ ((aligned (64))); /* receive BD ring */
TSEC_TXBUFFER			transmitBuffer[NUM_OF_BD_BUFFER] __attribute__ ((aligned (64)));/* transmit buffer array */


/* forward declaration */

void prepareTxbufferBd (void);
void prepareRxbufferBd (void);
void tsecInit (void);
void resetMac (void);
void assignMacAttributes (void);
void assignAddresses (void);
void assignMacReceiverAttributes (void);
void setupTsecInterrupt (void);
void setupTsecDmaController (void);
void enableReceiverTransmitter (void);
unsigned long	checkResults (void);


/******************************************************************
* tsecInit - calls other subroutines and intializes tsec and phy
*
* This routine calls other subroutines to initialize the mac
* controller of TSEC. If user selects TSEC PHY digital loopback then
* it also initializes the PHY. This routine calls subroutine to 
* setup the management interface.
*
* RETURNS: None
*      
* ERRNO: None 
*
*/

void tsecInit ()
{
	unsigned long status;


	/* Reset the mac controller */

    resetMac();
	
	/* Setup the mac controller attributes */

	assignMacAttributes ();

	                        
	/* 
	* Station address is assigned. The group and
	* individual address fields are cleared out 
	* indicating that this test code is capable of 
	* decoding station address for the receiver. 
	*/
	
	assignAddresses ();

	/*
	* PHY is intialized if the digital phy loopback 
	* or normal mode is selected. For mac internal loopback
	* there is no need to initialize PHY
	*/
	
	#ifndef MAC_INTERNAL_LOOPBACK
		
		phyInit();
	
	#endif


	/* 
	* Set the receive buffer length register. Note that
	* upon power on reset this register will have a value 0. 
	* Therefore if this register is not programmed then the 
	* receiver will not be able to receive any data in the 
	* receive buffer. In the event when this register value 
	* is left cleared, BSY error will be generated. In this 
	* test example transmit buffer size is 256 bytes. On the 
	* receive side we provide 512 bytes for buffer. Note that 
	* we should also keep 4 bytes for receive CRC. 
	*/
	
	assignMacReceiverAttributes ();

	/* 
	* TSEC controller specific interrupt events are set.
	* Also EPIC is configured if needed
	*/

	setupTsecInterrupt ();

	/* 
	* Set the attributes for the TSEC DMA controller. Snooping
	* for the transmit and receive BDs are turned on. This will 
	* maintain coherency between the CPU and TSEC in terms of their
	* views about the BDs. Also for the transmit BDs polling option 
	* is chosen. Write with response is selected to maitain the ordering
	* of updating BD status with that of reporting an event in the IEVENT
	* register.
	*/
	
	setupTsecDmaController ();
	
	/*
	* Prepare the BDs and buffers for both transmit 
	* and receive.
	*/

	prepareTxbufferBd ();
	prepareRxbufferBd ();


	/*
	* Enable the receiver and transmitter. If the mac loopback is 
	* requested then also turn the mac loopback bit. 
	*/
	enableReceiverTransmitter ();
	
	/* 
	* Check to see whether receive is completed. This checking
	* is performed only when we are doing the internal mac loopback
	* or PHY loopback. In a normal operation when we are transmitting
	* to the wire and receiving from wires we skip this checking
	*/
	#ifdef STATUS_POLLING
	#ifndef NORMAL_MODE
	
		status = checkResults ();
	
	#endif
	#endif
	
	/* 
	* Come in a tight loop and do nothing.
	* If interrupt scheme is selected for checking
	* the results then the handler will process 
	* information 
	*/
			
	while (1) 
	{
		
	}
}


/******************************************************************
* prepareTxbufferBd - sets up transmit BDs and buffers
*
* This routine initializes four BDs in the transmit BD ring. For
* each of the BDs, length and pointer fields are updated. Also 
* the R-bits of the BDs are set, so that when the mac controller
* is enabled for transmit data from the transmit buffers will be 
* transmitted.
*
* RETURNS: None
*      
* ERRNO: None 
*
*/
void prepareTxbufferBd (void)
{
	unsigned char *pbyte;
	unsigned long i;
	unsigned long j;
	unsigned long mem_value;
	
	/* 	Prepare the transmit BDs and marked them ready */

	for (i =0; i < NUM_OF_BD_BUFFER; i++)
		{
		if ((i == (NUM_OF_BD_BUFFER - 1)))
		{
			#ifdef STATUS_POLLING
				transmitBd[i].flags  = (0xa000 | 0x0800);
			#endif

			#ifdef STATUS_INTERRUPT
				transmitBd[i].flags  = (0xb000 | 0x0800);
			#endif			
		}
		else
		{
			#ifdef STATUS_POLLING
				transmitBd[i].flags  = (0x8000 | 0x0800);	
			#endif
					
			/* 
			* Generate interrrupt only for the last txbd
			* and not for all the intermediate ones 
			*/
			#ifdef STATUS_INTERRUPT
				transmitBd[i].flags  = (0x9000 | 0x0800);	
				//transmitBd[i].flags  = (0x8000 | 0x0800);	
			#endif
		}
			
		
		transmitBd[i].length = TX_BUFFER_SIZE;
		transmitBd[i].Buffer = (unsigned long)(&transmitBuffer[i]);		
		}
	
	/* 	
	* Initialize the transmit buffers. Note that the 
	* mac address (source and destination) and the length are
	* all part of the transmit buffer. Here the address and 
	* length fields are initialized
	*/

	for (i =0; i < NUM_OF_BD_BUFFER; i++)
		{
		transmitBuffer[i].DestAddr1    = 0x12345678;
		transmitBuffer[i].DestAddr2    = 0xABCD;
		transmitBuffer[i].SourceAddr1  = 0xaabbccdd;
		transmitBuffer[i].SourceAddr2  = 0xeeff;	
		transmitBuffer[i].LengthOrType = (TX_BUFFER_SIZE - ETHERNET_HDR_SIZE); 			
		}
	
	/* Initialize the data portion of the transmit buffers */

	for (j = 0; j < NUM_OF_BD_BUFFER; j++)
		{
		mem_value = 0x0;
		pbyte = (unsigned char *)((unsigned long)&transmitBuffer[j] + (unsigned long)(ETHERNET_HDR_SIZE));
		for (i =0; i < 242; i++)
			*pbyte++ = mem_value++ % 0xff;
		}
	
	/* 
	* Set the TBASE register. TBASE points to the start of
	* the transmit BD ring. Note that at least two BDs should 
	* be present in a transmit or receive BD ring
	*/

	pTsecMap->tbase = (reg32)(&transmitBd[0]);
}

/***************************************************************
* prepareRxbufferBd - sets up transmit BDs and buffers
*
* This routine initializes four BDs in the receive BD ring. For
* each of the BDs, length and pointer fields are updated. Also 
* the E-bits of the BDs are set, so that when the mac controller
* is enabled for receive data, the data will be collected in
* the receive buffers pointed to by these BDs.
*
* RETURNS: None
*      
* ERRNO: None 
*
*/

void prepareRxbufferBd (void)
{
	unsigned long *pWord;
	unsigned long i;
	unsigned long j;
	
	/* Prepare the receive BDs and marked them empty */

	for (i =0; i < NUM_OF_BD_BUFFER; i++)
		{
		if ((i == (NUM_OF_BD_BUFFER - 1)))
		{
			#ifdef STATUS_POLLING
				receiveBd[i].flags  = 0xa000;
			#endif
			
			#ifdef STATUS_INTERRUPT
				receiveBd[i].flags  = 0xb000;
			#endif			
			
		}
			
		else
		{
			#ifdef STATUS_POLLING
				receiveBd[i].flags  = 0x8000;	
			#endif
			
			#ifdef STATUS_INTERRUPT
				receiveBd[i].flags  = 0x9000;	
			#endif			
		}
			
		
		receiveBd[i].Buffer = (RX_BUFFER_LOCATION + (i * RX_BUFFER_SIZE));		
		}
	
	/* 	
	* Initialize the RxBuffers. Zero out all the receive
	* buffers. This way it is easier to verify whether 
	* TSEC receiver received data and comparison can be 
	* made between the transmit and receive buffer
	*/
	
	for (j = 0; j < NUM_OF_BD_BUFFER; j++)
		{
		pWord = (unsigned long *)(RX_BUFFER_LOCATION + j*RX_BUFFER_SIZE);
		for (i =0; i < (RX_BUFFER_SIZE / 4); i++)
			*pWord++ = 0x0;
		}
		
	
	/* 
	* Set the RBASE register. RBASE points to the start of
	* the receive BD ring. Note that at least two BDs should 
	* be present in a transmit or receive BD ring
	*/
	
	pTsecMap->rbase = (reg32)(&receiveBd[0]);
	
}

void resetMac (void)
{
	pTsecMap->maccfg1     |= MACCFG1_SOFT_RESET;
	pTsecMap->maccfg1     &= ~MACCFG1_SOFT_RESET;
}


void 	assignMacAttributes (void)
{
	pTsecMap->maccfg2     |= (MACCFG2_PAD_CRC    | 
	                          MACCFG2_CRC_ENABLE ); 

	/* Select controller interface based on user input */
	
	#if (INTERFACE_MODE == GMII_INTERFACE )
			pTsecMap->maccfg2     |= (MACCFG2_GMII_TBI_MODE | MACCFG2_FULL_DUPLEX ); 	                        		
	#endif

	#if (INTERFACE_MODE == TBI_INTERFACE )
			pTsecMap->maccfg2     |= (MACCFG2_GMII_TBI_MODE | MACCFG2_FULL_DUPLEX); 	                        		
	#endif

	#if (INTERFACE_MODE == MII_INTERFACE  )
			pTsecMap->maccfg2     |= (MACCFG2_MII_MODE | MACCFG2_FULL_DUPLEX); 	                        		
	#endif

	
}


void assignAddresses (void)
{
	unsigned long i;

	/* 
	* Assign station address. Note that the station address
	* should be written in little endian format. If you intend 
	* to assign a mac address 0x12345678ABCD, then you byte swap
	* and write the values to the two station address registers
	*/

	pTsecMap->macstnaddr1 = 0xCDAB7856;
	pTsecMap->macstnaddr2 = 0x34120000;

	/* Clear all the individual addr registers */

	for(i = 0; i < 8; i++)
		pTsecMap->iaddr[i] = 0x0;	


	/* Clear all the group addr registers */
	
	for(i = 0; i < 8; i++)
		pTsecMap->gaddr[i] = 0x0;

}

void assignMacReceiverAttributes (void)
{
	pTsecMap->mrblr |= (RX_BUFFER_SIZE + 64);
	//pTsecMap->rctrl	  = 0x0;				
	//pTsecMap->rctrl	  |= RCTRL_PROM;				
	
}

void setupTsecInterrupt (void)
{
	/* Clear all bogus events from the IEVENT register */
	
	pTsecMap->ievent = 0xffffffff;
	
	
	#ifdef STATUS_POLLING
		/* 
		* We don't have interrupt scheme implemented. Thus mask
		* out all interrupts in IMASK register. Later when interrupts
		* will be implemented we can program the IMASK register to 
		* an appropriate value to unmask the events that we want.
		*/
		pTsecMap->imask  = 0x0;
	#endif
	
	#ifdef STATUS_INTERRUPT

		/* Enable the events that we need to capture */
		
		pTsecMap->imask = (IMASK_TXBEN | IMASK_TXFEN | IMASK_RXFEN | IMASK_RXBEN);
		
		/* Provision for transmit interrupt coalesing */
		pTsecMap->txic  |= (0x80000000 | ((NUM_OF_BD_BUFFER / 2 ) << 21 ) | 0x0000ffff);		
		
		/* Provision for receive interrupt coalesing */
		pTsecMap->rxic |= (0x80000000 | ((NUM_OF_BD_BUFFER / 2 ) << 21 ) | 0x0000ffff);		
		

	#endif	
}


void 	setupTsecDmaController (void)
{
	pTsecMap->dmactrl |= (DMACTRL_TDSEN | DMACTRL_TBDSEN | DMACTRL_WWR);
}

void 	enableReceiverTransmitter (void)
{
	#ifdef PHY_LOOPBACK
		pTsecMap->maccfg1   |= (MACCFG1_RXEN | MACCFG1_TXEN); 
	#endif
	
	#ifdef PHY_CONNECTOR_LOOPBACK
		pTsecMap->maccfg1   |= (MACCFG1_RXEN | MACCFG1_TXEN); 
	#endif
	

	#ifdef NORMAL_MODE
		pTsecMap->maccfg1   |= (MACCFG1_RXEN | MACCFG1_TXEN); 
	#endif	
	
	#ifdef MAC_INTERNAL_LOOPBACK
		pTsecMap->maccfg1   |= (MACCFG1_LOOPBACK | MACCFG1_RXEN | MACCFG1_TXEN); 
	#endif
	
	/* Wait for transmitter and receiver activation indication */
	
	while (1)
		{
		if ((pTsecMap->maccfg1 & MACCFG1_SYNC_TXEN) && (pTsecMap->maccfg1 & MACCFG1_SYNC_RXEN))
			break;
		}
	
}


unsigned long	checkResults (void)
{
	unsigned long i;
	unsigned long j;
	unsigned char *pSource;
	unsigned char *pDestination;
	unsigned long status = 0x0;

	while (1)
		{
		j = 0;
		for (i =0; i < NUM_OF_BD_BUFFER; i++)
			{		
			if (receiveBd[i].flags & 0x8000)
				j++;
			}	
		if ( j == NUM_OF_BD_BUFFER)
			break;
		}
		
	/* Compare the receive data with transmitted data */
	
	j = 0;
	
	for (i =0; i < NUM_OF_BD_BUFFER; i++)
		{
		pSource = (unsigned char *)(unsigned long)&transmitBuffer[i] + (unsigned long)(ETHERNET_HDR_SIZE);
		pDestination = (unsigned char *)((RX_BUFFER_LOCATION + i*RX_BUFFER_SIZE) + (unsigned long)(ETHERNET_HDR_SIZE));
		if (!memcmp (pSource, pDestination, ( TX_BUFFER_SIZE - ETHERNET_HDR_SIZE)))
			j++;
		}
	if ( j == NUM_OF_BD_BUFFER)
		status = 0xffffffff;
	else
		status = 0x00000000;
	
	return status;
	
}